/****************************************/
/* This is the checkset.c module for 	*/
/* lsat program. This module checks for */
/* setuid and setgid files on the 	*/
/* local system and prints them to the  */
/* outfile.                             */
/*					*/
/* See lsatmain.c for more details.	*/
/****************************************/

#include <stdio.h>
#include <stdlib.h>

checkset(filename)
const char *const filename;
{

	char line[80];  /* for reading lines		*/
	FILE *outfile; 	/* file pointer for outfile	*/
	FILE *fileptr;	/* file pointer for infile	*/

	/* 1st off generate a list of setuid files 	*/
	/* this should be done from /, but use -xdev so	*/
	/* that we don't traverse other filesystems	*/			
	/* note, we use -mount for compatibility with 	*/
	/* other versions of find.			*/
	
	if (system("find / -mount -perm -4000 1>/dev/null 2>/dev/null >/tmp/findlist.txt") == 2)
	{
		perror("Problem writing file to tmp.\n");
		perror("Problem was in module: checkset.c\n");
		return(-1);
	}

	/* now simply read through them and print them 	*/
	/* out to the outfile with a header.		*/

	/* open the outfile for append			*/
	outfile = fopen(filename, "a");
	if (outfile == NULL)
	{
		perror("Error opening outfile!\n");
		perror("Problem was in module: checkset.c\n");
		return(-1);
	}

	/* open the file with setuid files for read	*/
	fileptr = fopen("/tmp/findlist.txt", "r");
	if (fileptr == NULL)
	{
		perror("Error opening file for read!\n");
		perror("Problem was in module: checkset.c\n");
		return(-1);
	}
	
	/* write out a header to the outfile		*/
	fprintf(outfile, "****************************************\n");
	fprintf(outfile, "This is a list of SUID and SGID files\n");
	fprintf(outfile, "on the system. Keep track of them, and\n");
	fprintf(outfile, "consider changing perms or locking them\n");
	fprintf(outfile, "down...\n");
	fprintf(outfile, "Search google or sans for more info.\n");
	fprintf(outfile, "**********\n");
	fprintf(outfile, "\nSetUID files found:\n");

	/* ok, now just print them out...		*/
	while (fgets(line, sizeof(line), fileptr) != NULL)
	{
		fprintf(outfile, "%s\n", line);
	}
	
	/* close the input file and rm it		*/
	fclose(fileptr);
	if ((system("rm -f /tmp/findlist.txt")) < 0)
	{
		/* rhut-rho... problems.. */
		perror("Can't rm tempfile...\n");
		perror("Permissions problem??\n");
		return(-1);
	}

	/* get a list of setgid files 			*/
	if (system("find / -mount -perm -2000 1>/dev/null 2>/dev/null >/tmp/findlist.txt") == 2)
        {
                perror("Problem writing file to tmp.\n");
                perror("Problem was in module: checkset.c\n");
                return(-1);
        }

	 /* open the file with setgid files for read     */
        fileptr = fopen("/tmp/findlist.txt", "r");
        if (fileptr == NULL)
        {
                perror("Error opening file for read!\n");
                perror("Problem was in module: checkset.c\n");
                return(-1);
        }

	/* print a small header				*/
	fprintf(outfile, "\n**********\n");
	fprintf(outfile, "SetGID files found:\n");

	/* ok, now just print them out...               */
        while (fgets(line, sizeof(line), fileptr) != NULL)
        {
                fprintf(outfile, "%s\n", line);
        }

        /* close the input file and rm it               */
        fclose(fileptr);
        if ((system("rm -f /tmp/findlist.txt")) < 0)
        {
                /* rhut-rho... problems.. */
                perror("Can't rm tempfile...\n");
                perror("Permissions problem??\n");
                return(-1);
        }
	 
	/* print out a footer... */
	fprintf(outfile, "\n");
	fprintf(outfile, "****************************************\n");
	
	/* close up */
	fclose(outfile);
	
	return(0);
}		
